/*
  Control32Servo.cpp - Library for controlling Siku Control 32 Servos.
  Created by Oisin O'Conchubhair, November 29, 2015.
  Based on the servoTimer2 library.
  Released into the public domain.
*/

#include "Arduino.h"
#include <avr/interrupt.h>
#include "Control32Servo.h"

static void initISR();

static Control32_Servo_t servos[NBR_CHANNELS+1];    // static array holding servo data for all channels

static volatile uint8_t Channel=0;   // counter holding the channel being pulsed
static volatile uint8_t ISRCount;  // iteration counter used in the interrupt routines;
uint8_t ChannelCount = 0;          // counter holding the number of attached channels
static boolean isStarted = false;  // flag to indicate if the ISR has been initialised
static int Active_Pulse_Width = 0;
static int Pulse_State = 0;
static int Bit_Num = 0;
static int On_Width = 6;
static int Off_Width = 2;
static int Gap_Width = 70;




ISR (TIMER2_OVF_vect)
{
	if(Bit_Num==8 && Pulse_State==1){
					digitalWrite(servos[Channel].Pin.nbr,HIGH);
		if(ISRCount==18){

			Channel++;
			if(Channel==NBR_CHANNELS+1){
				Channel=1;
			}
			Active_Pulse_Width=servos[Channel].Pulsewidth<<1;
			bitSet(Active_Pulse_Width,0);

			ISRCount=0;
			Bit_Num=0;
			Pulse_State=0;
		}
		else{
			++ISRCount; // increment the overlflow counter
			TCNT2 = 255-(255);
		}
	}
	else if(bitRead(Active_Pulse_Width,Bit_Num)==1 && Pulse_State==0){
		digitalWrite(servos[Channel].Pin.nbr,LOW);
		TCNT2 = 256-(2*On_Width);    // reset the clock counter register
		Pulse_State=1;
	}
	else if(bitRead(Active_Pulse_Width,Bit_Num)==0 && Pulse_State==0){
		digitalWrite(servos[Channel].Pin.nbr,LOW);
		TCNT2 = 256-(2*Off_Width);    // reset the clock counter register
		Pulse_State=1;
	}
	else if(Bit_Num!=8 && Pulse_State==1){
		TCNT2 = 256-(2*Gap_Width);    // reset the clock counter register
		if(ISRCount==1){
			Pulse_State=0;
			ISRCount=0;
			Bit_Num++;
		}
		else{
			digitalWrite(servos[Channel].Pin.nbr,HIGH);
			++ISRCount; // increment the overlflow counter
		}
	}


}



Control32Servo::Control32Servo()
{


  if( ChannelCount < NBR_CHANNELS)
     this->chanIndex = ++ChannelCount;  // assign a channel number to this instance
  else
     this->chanIndex = 0;  // todo      // too many channels, assigning 0 inhibits this instance from functioning
}



uint8_t Control32Servo::attach(int pin)
{
     if( isStarted == false)
      initISR();
     if(this->chanIndex > 0)
     {
      //debug("attaching chan = ", chanIndex);
      pinMode( pin, OUTPUT) ;  // set servo pin to output
      servos[this->chanIndex].Pin.nbr = pin;
      servos[this->chanIndex].Pin.isActive = true;
     }
     return this->chanIndex ;
}


void Control32Servo::detach()
{

   servos[this->chanIndex].Pin.isActive = false;
}


void Control32Servo::write(int pulsewidth)
{
	if(pulsewidth>180){
		pulsewidth=180;
	}
	  pulsewidth = map(pulsewidth, 0, 180, 0, 127);     // scale it to use it with the servo (value between 0 and 180)
servos[this->chanIndex].Pulsewidth = pulsewidth;
}


boolean Control32Servo::attached()
{
   return servos[this->chanIndex].Pin.isActive ;
}


static void initISR()
{
     Channel = 0;  // clear the channel index
     ISRCount = 0;  // clear the value of the ISR counter;

     /* setup for timer 2 */
     TIMSK2 = 0;  // disable interrupts
     TCCR2A = 0;  // normal counting mode
     TCCR2B = 0;
     TCCR2B=(1<<CS21);

     TCNT2 = 0;     // clear the timer2 count
     TIFR2 = _BV(TOV2);  // clear pending interrupts;
     TIMSK2 =  _BV(TOIE2) ; // enable the overflow interrupt

     isStarted = true;  // flag to indicate this initialisation code has been executed
}